@@ -1,7 +1,8 @@
 /*	$OpenBSD: exec.c,v 1.49 2009/01/29 23:27:26 jaredy Exp $	*/
 
 /*-
- * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+ * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
+ *		 2011, 2012
  *	Thorsten Glaser <tg@mirbsd.org>
  *
  * Provided that these terms and disclaimer and all copyright notices
@@ -22,7 +23,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/exec.c,v 1.96 2011/09/07 15:24:14 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/exec.c,v 1.99 2012/06/24 20:05:23 tg Exp $");
 
 #ifndef MKSH_DEFAULT_EXECSHELL
 #define MKSH_DEFAULT_EXECSHELL	"/bin/sh"
@@ -234,8 +235,7 @@ execute(struct op * volatile t,
 		 */
 		sigprocmask(SIG_BLOCK, &sm_sigchld, &omask);
 		e->type = E_ERRH;
-		i = sigsetjmp(e->jbuf, 0);
-		if (i) {
+		if ((i = kshsetjmp(e->jbuf))) {
 			sigprocmask(SIG_SETMASK, &omask, NULL);
 			quitenv(NULL);
 			unwind(i);
@@ -337,10 +337,7 @@ execute(struct op * volatile t,
 		    (const char **)eval((const char **)t->vars,
 		    DOBLANK | DOGLOB | DOTILDE);
 		e->type = E_LOOP;
-		while (/* CONSTCOND */ 1) {
-			i = sigsetjmp(e->jbuf, 0);
-			if (!i)
-				break;
+		while ((i = kshsetjmp(e->jbuf))) {
 			if ((e->flags&EF_BRKCONT_PASS) ||
 			    (i != LBREAK && i != LCONTIN)) {
 				quitenv(NULL);
@@ -375,10 +372,7 @@ execute(struct op * volatile t,
 	case TWHILE:
 	case TUNTIL:
 		e->type = E_LOOP;
-		while (/* CONSTCOND */ 1) {
-			i = sigsetjmp(e->jbuf, 0);
-			if (!i)
-				break;
+		while ((i = kshsetjmp(e->jbuf))) {
 			if ((e->flags&EF_BRKCONT_PASS) ||
 			    (i != LBREAK && i != LCONTIN)) {
 				quitenv(NULL);
@@ -733,8 +727,7 @@ comexec(struct op *t, struct tbl * volatile tp, const char **ap,
 		tp->flag |= FINUSE;
 
 		e->type = E_FUNC;
-		i = sigsetjmp(e->jbuf, 0);
-		if (i == 0) {
+		if (!(i = kshsetjmp(e->jbuf))) {
 			/* seems odd to pass XERROK here, but AT&T ksh does */
 			exstat = execute(tp->val.t, flags & XERROK, xerrok);
 			i = LRETURN;
@@ -1271,7 +1264,8 @@ iosetup(struct ioword *iop, struct tbl *tp)
 	int u = -1;
 	char *cp = iop->name;
 	int iotype = iop->flag & IOTYPE;
-	int do_open = 1, do_close = 0, flags = 0;
+	bool do_open = true, do_close = false;
+	int flags = 0;
 	struct ioword iotmp;
 	struct stat statb;
 
@@ -1313,7 +1307,7 @@ iosetup(struct ioword *iop, struct tbl *tp)
 		break;
 
 	case IOHERE:
-		do_open = 0;
+		do_open = false;
 		/* herein() returns -2 if error has been printed */
 		u = herein(iop->heredoc, iop->flag & IOEVAL, NULL);
 		/* cp may have wrong name */
@@ -1322,11 +1316,11 @@ iosetup(struct ioword *iop, struct tbl *tp)
 	case IODUP: {
 		const char *emsg;
 
-		do_open = 0;
+		do_open = false;
 		if (*cp == '-' && !cp[1]) {
 			/* prevent error return below */
 			u = 1009;
-			do_close = 1;
+			do_close = true;
 		} else if ((u = check_fd(cp,
 		    X_OK | ((iop->flag & IORDUP) ? R_OK : W_OK),
 		    &emsg)) < 0) {
@@ -1338,7 +1332,7 @@ iosetup(struct ioword *iop, struct tbl *tp)
 			/* "dup from" == "dup to" */
 			return (0);
 		break;
-	}
+	    }
 	}
 
 	if (do_open) {
@@ -1424,7 +1418,7 @@ hereinval(const char *content, int sub, char **resbuf, struct shf *shf)
 
 	osource = source;
 	newenv(E_ERRH);
-	if (sigsetjmp(e->jbuf, 0)) {
+	if (kshsetjmp(e->jbuf)) {
 		source = osource;
 		quitenv(shf);
 		/* special to iosetup(): don't print error */
@@ -1475,10 +1469,10 @@ herein(const char *content, int sub, char **resbuf)
 	 * so temp doesn't get removed too soon).
 	 */
 	h = maketemp(ATEMP, TT_HEREDOC_EXP, &e->temps);
-	if (!(shf = h->shf) || (fd = open(h->name, O_RDONLY, 0)) < 0) {
+	if (!(shf = h->shf) || (fd = open(h->tffn, O_RDONLY, 0)) < 0) {
 		i = errno;
 		warningf(true, "can't %s temporary file %s: %s",
-		    !shf ? "create" : "open", h->name, strerror(i));
+		    !shf ? "create" : "open", h->tffn, strerror(i));
 		if (shf)
 			shf_close(shf);
 		/* special to iosetup(): don't print error */
@@ -1494,7 +1488,8 @@ herein(const char *content, int sub, char **resbuf)
 	if (shf_close(shf) == EOF) {
 		i = errno;
 		close(fd);
-		warningf(true, "%s: %s: %s", "write", h->name, strerror(i));
+		warningf(true, "can't %s temporary file %s: %s",
+		    "write", h->tffn, strerror(i));
 		/* special to iosetup(): don't print error */
 		return (-2);
 	}
