@@ -1,7 +1,8 @@
 /*	$OpenBSD: expr.c,v 1.21 2009/06/01 19:00:57 deraadt Exp $	*/
 
 /*-
- * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+ * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
+ *		 2011, 2012
  *	Thorsten Glaser <tg@mirbsd.org>
  *
  * Provided that these terms and disclaimer and all copyright notices
@@ -22,7 +23,21 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/expr.c,v 1.51 2011/12/11 01:56:43 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/expr.c,v 1.58 2012/06/28 20:17:36 tg Exp $");
+
+#if !HAVE_SILENT_IDIVWRAPV
+#if !defined(MKSH_LEGACY_MODE) || HAVE_LONG_32BIT
+#define IDIVWRAPV_VL	(mksh_uari_t)0x80000000UL
+#define IDIVWRAPV_VR	(mksh_uari_t)0xFFFFFFFFUL
+#elif HAVE_LONG_64BIT
+#define IDIVWRAPV_VL	(mksh_uari_t)0x8000000000000000UL
+#define IDIVWRAPV_VR	(mksh_uari_t)0xFFFFFFFFFFFFFFFFUL
+#else
+# warning "cannot guarantee integer division wraparound"
+#undef HAVE_SILENT_IDIVWRAPV
+#define HAVE_SILENT_IDIVWRAPV 1
+#endif
+#endif
 
 /* The order of these enums is constrained by the order of opinfo[] */
 enum token {
@@ -159,7 +174,6 @@ static struct tbl *evalexpr(Expr_state *, int);
 static void exprtoken(Expr_state *);
 static struct tbl *do_ppmm(Expr_state *, enum token, struct tbl *, bool);
 static void assign_check(Expr_state *, enum token, struct tbl *);
-static struct tbl *tempvar(void);
 static struct tbl *intvar(Expr_state *, struct tbl *);
 
 /*
@@ -198,8 +212,7 @@ v_evaluate(struct tbl *vp, const char *expr, volatile int error_ok,
 	curstate.natural = false;
 
 	newenv(E_ERRH);
-	i = sigsetjmp(e->jbuf, 0);
-	if (i) {
+	if ((i = kshsetjmp(e->jbuf))) {
 		/* Clear EXPRINEVAL in of any variables we were playing with */
 		if (curstate.evaling)
 			curstate.evaling->flag &= ~EXPRINEVAL;
@@ -284,7 +297,7 @@ evalerr(Expr_state *es, enum error_type type, const char *str)
 
 	case ET_RDONLY:
 		warningf(true, "%s: %s %s",
-		    es->expression, str, "applied to read only variable");
+		    es->expression, str, "applied to read-only variable");
 		break;
 
 	default: /* keep gcc happy */
@@ -347,7 +360,8 @@ evalexpr(Expr_state *es, int prec)
 			/* vl may not have a value yet */
 			vl = intvar(es, vl);
 		if (IS_ASSIGNOP(op)) {
-			assign_check(es, op, vasn);
+			if (!es->noassign)
+				assign_check(es, op, vasn);
 			vr = intvar(es, evalexpr(es, P_ASSIGN));
 		} else if (op != O_TERN && op != O_LAND && op != O_LOR)
 			vr = intvar(es, evalexpr(es, prec - 1));
@@ -365,21 +379,35 @@ evalexpr(Expr_state *es, int prec)
 			break;
 		case O_DIV:
 		case O_DIVASN:
-			if (!es->natural && vr->val.i == -1 &&
-			    vl->val.i == ((mksh_ari_t)1 << 31)) {
+#if !HAVE_SILENT_IDIVWRAPV
+			/*
+			 * we are doing the comparisons here for the
+			 * signed arithmetics (!es->natural) case,
+			 * but the exact value checks and the bypass
+			 * case assignments are done unsignedly as
+			 * several compilers bitch around otherwise
+			 */
+			if (!es->natural &&
+			    vl->val.u == IDIVWRAPV_VL &&
+			    vr->val.u == IDIVWRAPV_VR) {
 				/* -2147483648 / -1 = 2147483648 */
-				/* 80000000 / FFFFFFFF = 80000000 */
-				res = ((mksh_ari_t)1 << 31);
+				/* this ^ is really (1 << 31) though */
+				res = (mksh_ari_t)IDIVWRAPV_VL;
 			} else
+#endif
 				res = bivui(vl, /, vr);
 			break;
 		case O_MOD:
 		case O_MODASN:
-			if (!es->natural && vr->val.i == -1 &&
-			    vl->val.i == ((mksh_ari_t)1 << 31)) {
+#if !HAVE_SILENT_IDIVWRAPV
+			/* see O_DIV / O_DIVASN for the reason behind this */
+			if (!es->natural &&
+			    vl->val.u == IDIVWRAPV_VL &&
+			    vr->val.u == IDIVWRAPV_VR) {
 				/* -2147483648 % -1 = 0 */
 				res = 0;
 			} else
+#endif
 				res = bivui(vl, %, vr);
 			break;
 		case O_PLUS:
@@ -609,7 +637,8 @@ do_ppmm(Expr_state *es, enum token op, struct tbl *vasn, bool is_prefix)
 		setint_v(vasn, vl, es->arith);
 	else
 		setint(vasn, vl->val.i);
-	if (!is_prefix)		/* undo the inc/dec */
+	if (!is_prefix)
+		/* undo the increment/decrement */
 		vl->val.i = oval;
 
 	return (vl);
@@ -625,7 +654,7 @@ assign_check(Expr_state *es, enum token op, struct tbl *vasn)
 		evalerr(es, ET_RDONLY, opinfo[(int)op].name);
 }
 
-static struct tbl *
+struct tbl *
 tempvar(void)
 {
 	struct tbl *vp;
