@@ -4,7 +4,7 @@
 /*	$OpenBSD: tree.h,v 1.10 2005/03/28 21:28:22 deraadt Exp $	*/
 /*	$OpenBSD: expand.h,v 1.6 2005/03/30 17:16:37 deraadt Exp $	*/
 /*	$OpenBSD: lex.h,v 1.11 2006/05/29 18:22:24 otto Exp $	*/
-/*	$OpenBSD: proto.h,v 1.33 2010/05/19 17:36:08 jasper Exp $	*/
+/*	$OpenBSD: proto.h,v 1.34 2012/06/27 07:17:19 otto Exp $	*/
 /*	$OpenBSD: c_test.h,v 1.4 2004/12/20 11:34:26 otto Exp $	*/
 /*	$OpenBSD: tty.h,v 1.5 2004/12/20 11:34:26 otto Exp $	*/
 
@@ -157,9 +157,9 @@
 #endif
 
 #ifdef EXTERN
-__RCSID("$MirOS: src/bin/mksh/sh.h,v 1.564 2012/06/26 19:33:33 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/sh.h,v 1.568 2012/06/28 20:17:37 tg Exp $");
 #endif
-#define MKSH_VERSION "R40 2012/06/26"
+#define MKSH_VERSION "R40 2012/06/28"
 
 /* arithmetic types: C implementation */
 #if !HAVE_CAN_INTTYPES
@@ -172,16 +172,21 @@ typedef u_int32_t uint32_t;
 #endif
 
 /* arithmetic types: shell arithmetics */
+#ifdef MKSH_LEGACY_MODE
 /*
- * NOTE: these are currently hard-coded to exactly 32 bit, do not change
- *
- * TODO: make these configurable, or add 64-bit arithmetic, somehow
- * (on some operating environments, 64-bit types are needed for some
- * things, such as the ulimit builtin, to work proper), except mksh
- * must still be able to run on systems with no native 64-bit integers
+ * POSIX demands these to be the C environment's long type
+ */
+typedef long mksh_ari_t;
+typedef unsigned long mksh_uari_t;
+#else
+/*
+ * These types are exactly 32 bit wide; signed and unsigned
+ * integer wraparound, even across division and modulo, for
+ * any shell code using them, is guaranteed.
  */
 typedef int32_t mksh_ari_t;
 typedef uint32_t mksh_uari_t;
+#endif
 
 /* boolean type (no <stdbool.h> deliberately) */
 typedef unsigned char mksh_bool;
@@ -1063,6 +1068,8 @@ struct tbl {
 	char name[4];
 };
 
+EXTERN struct tbl vtemp;
+
 /* common flag bits */
 #define ALLOC		BIT(0)	/* val.s has been allocated */
 #define DEFINED		BIT(1)	/* is defined in block */
@@ -1460,6 +1467,7 @@ struct source {
 #define SWORDSEP	6	/* string[] separator */
 #define SALIAS		7	/* alias expansion */
 #define SREREAD		8	/* read ahead to be re-scanned */
+#define SSTRINGCMDLINE	9	/* string from "mksh -c ..." */
 
 /* Source.flags values */
 #define SF_ECHO		BIT(0)	/* echo input to shlout */
@@ -1643,6 +1651,7 @@ size_t utf_ptradj(const char *);
 int utf_wcwidth(unsigned int);
 #endif
 int ksh_access(const char *, int);
+struct tbl *tempvar(void);
 /* funcs.c */
 int c_hash(const char **);
 int c_pwd(const char **);
